/*! Tripetto Collector 1.7.1-59 - Copyright (C) 2020 Tripetto B.V. - All Rights Reserved */

/// <reference path="marblecore.l10n.d.ts" />

declare module "@tripetto/contracts" {
    /** Exports */
    export * from "@tripetto/contracts/l10n";
    export * from "@tripetto/contracts/styles";
    export * from "@tripetto/contracts/themes";
}

declare module "@tripetto/contracts/l10n" {
    import { TTranslation } from "@marblecore/l10n";
    export interface IL10nContract {
        /** Specifies the language identifiers of the available translations. */
        readonly availableTranslations?: string[];
        /** Specifies if the locale can be changed. */
        readonly customLocale?: boolean;
        /** Contains the translatable strings table. */
        readonly strings?: (string | [string, string])[];
    }
    export interface IL10n {
        /** Specifies the language of the translation to use. */
        readonly language?: "auto" | string;
        /** Specifies the locale to use. */
        readonly locale?: "auto" | string;
        /** Contains the translations. */
        readonly translations?: TTranslation | TTranslation[];
    }
}

declare module "@tripetto/contracts/styles" {
    export interface IStyleDependency {
        /** Specifies the property to depend on. */
        readonly property: string;
        /** Specifies the value that should be evaluated. */
        readonly value: string | number | boolean | undefined;
        /** Specifies the action to perform. */
        readonly action: "enable" | "disable" | "show" | "hide";
    }
    export type TStyleTiers = "standard" | "premium";
    export interface IStyleStatic<Type extends string> {
        /** Specifies the style element type. */
        readonly type: Type;
        /** Specifies the variable and its value where the state of this style element depends on. */
        readonly dependency?: IStyleDependency;
        /** Specifies a label for the style element. */
        readonly label: string;
        /** Specifies a optional description for the style element. */
        readonly description?: string;
        /** Specifies the required tier for using this style element. */
        readonly tier?: TStyleTiers;
    }
    export interface IStyleElement<Type extends string, T>
        extends IStyleStatic<Type> {
        /** Specifies the name of the style element property. */
        readonly name: string;
        /** Specifies the default value for the style element. */
        readonly default?: T;
    }
    export type TStyleDropdownOption =
        | {
              /** Label for the option. */
              readonly label: string;
              /** Value for the option. */
              readonly value: string | number;
          }
        | {
              /** Specifies an option group. */
              readonly optGroup: string;
          }
        | {
              /** Specifies a custom option that allows the user to specify a custom value. */
              readonly custom: string;
          };
    export interface IStyleDropdown extends IStyleElement<"dropdown", string> {
        /** Specifies the dropdown options. */
        readonly options: TStyleDropdownOption[];
        /** Specifies the width of the field. */
        readonly width?: number | "auto" | "full";
    }
    export interface IStyleRadiobutton {
        /** Label for the button. */
        readonly label: string;
        /** Value for the button. */
        readonly value: string | number;
        /** Optional description for the button. */
        readonly description?: string;
    }
    export interface IStyleRadiobuttons
        extends IStyleElement<"radiobuttons", string> {
        /** Specifies the radio buttons. */
        readonly buttons: IStyleRadiobutton[];
    }
    export interface IStyleNumber extends IStyleElement<"number", number> {
        /** Prefix for the number. */
        readonly prefix?: string;
        /** Suffix for the number. */
        readonly suffix?: string;
        /** Specifies the minimum value. */
        readonly min?: number;
        /** Specifies the maximum value. */
        readonly max?: number;
        /** Specifies the width of the field. */
        readonly width?: number;
    }
    export type TStyleElement =
        | IStyleStatic<"static">
        | IStyleElement<"label" | "color" | "image", string>
        | IStyleElement<"checkbox", boolean>
        | IStyleNumber
        | IStyleDropdown
        | IStyleRadiobuttons;
    interface IStyleCollectionBase {
        /** Specifies a description for the collection of style elements. */
        readonly description?: string;
        /** Specifies a property name for the styles. */
        readonly name?: string;
        /** Specifies the variable and its value where the state of this collection depends on. */
        readonly dependency?: IStyleDependency;
        /** Specifies the required tier for using this collection. */
        readonly tier?: TStyleTiers;
        /** Specifies the style elements in this collection. */
        readonly elements: TStyleElement[];
    }
    export interface IStyleCollection extends IStyleCollectionBase {
        /** Specifies the title for the collection of style elements. */
        readonly title?: string;
        /** Specifies if the collection contains optional customizations (a checkbox will be shown before the collection to enable or disable it). */
        readonly optional?: false;
        /** Specifies if the collection contains additional options (they will be displayed at the end). */
        readonly additional?: boolean;
    }
    export interface IStyleCollectionOptional extends IStyleCollectionBase {
        /** Specifies the title for the collection of style elements. */
        readonly title: string;
        /** Specifies if the collection contains optional customizations (a checkbox will be shown before the collection to enable or disable it). */
        readonly optional: true;
        /** An optional collection cannot be additional. */
        readonly additional?: undefined | false;
    }
    export type TStylesContract = (
        | IStyleCollection
        | IStyleCollectionOptional
    )[];
    export interface IStyles {
        [property: string]: string | number | boolean | IStyles | undefined;
    }
    export {};
}

declare module "@tripetto/contracts/themes" {
    import { IStyles } from "@tripetto/contracts/styles";
    export interface IThemeContract<T extends IStyles = IStyles> {
        /** Contains the theme name. */
        readonly name: string;
        /** Contains an optional description for the theme. */
        readonly description?: string;
        /** Contains a preview image (as URL or base64 encoded data). */
        readonly preview?: string;
        /** Contains the styles for the theme. */
        readonly styles: T;
    }
    export type TThemesContract<T extends IStyles = IStyles> = IThemeContract<
        T
    >[];
}
